"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const fs = require("fs");
const os_1 = require("os");
const next = require("next");
const axios_1 = require("axios");
const component_1 = require("./react/component");
const FormData = require('form-data');
const util = require('util');
const exec = util.promisify(require('child_process').exec);
const { hashElement } = require('folder-hash');
const ora = require('ora');
const BOTONIC_CLIENT_ID = process.env.BOTONIC_CLIENT_ID || 'jOIYDdvcfwqwSs7ZJ1CpmTKcE7UDapZDOSobFmEp';
const BOTONIC_CLIENT_SECRET = process.env.BOTONIC_CLIENT_SECRET || 'YY34FaaNMnIVKztd6LbLIKn3wFqtiLhDgl6ZVyICwsLVWkZN9UzXw0GXFMmWinP3noNGU9Obtb6Nrr1BwMc4IlCTcRDOKJ96JME5N02IGnIY62ZUezMgfeiUZUmMSu68';
const BOTONIC_URL = process.env.BOTONIC_URL || 'https://api.hubtype.com';
class Botonic {
    constructor(config_path) {
        this.current_path = process.cwd();
        this.bot_path = path_1.join(this.current_path, '/.botonic.json');
        this.df_session_id = Math.random();
        this.React = { Component: component_1.Component };
        this.path = config_path;
        this.conf = require(path_1.join(this.path, '/.next/botonic.config.js'));
        process.chdir(this.path);
        this.app = next({ dev: false });
    }
    getAction(input) {
        let routeParams = {};
        //get ChildRoute depending of the past routepath
        let lastRouteChilds = this.getLastChildRoutes(this.lastRoutePath, this.conf.routes);
        if (lastRouteChilds) { //get route depending of current ChildRoute
            routeParams = this.getRoute(input, lastRouteChilds);
        }
        if (!routeParams || !Object.keys(routeParams).length) {
            /*
              we couldn't find a route in the state of the lastRoute, so let's find in
              the general conf.route
            */
            this.lastRoutePath = null;
            routeParams = this.getRoute(input, this.conf.routes);
        }
        if (!routeParams || !Object.keys(routeParams).length)
            return { action: '404', params: {} };
        if ('action' in routeParams.route) {
            if (this.lastRoutePath)
                this.lastRoutePath = `${this.lastRoutePath}/${routeParams.route.action}`;
            else
                this.lastRoutePath = routeParams.route.action;
            return { action: routeParams.route.action, params: routeParams.params };
        }
        else if ('redirect' in routeParams.route) {
            this.lastRoutePath = routeParams.route.redirect;
            let path = routeParams.route.redirect.split('/');
            return { action: path[path.length - 1], params: {} };
        }
        return { action: '404', params: {} };
    }
    getRoute(input, routes) {
        /*
          Find the input throw the routes, if it match with some of the entries,
          return the hole Route of the entry with optional params (used in regEx)
        */
        let params = {};
        let route = routes.find((r) => Object.entries(r)
            .filter(([key, value]) => key != 'action' || 'childRoutes')
            .some(([key, value]) => {
            let match = this.matchRoute(key, value, input);
            try {
                params = match.groups;
            }
            catch (e) { }
            return Boolean(match);
        }));
        if (route)
            return { route: route, params };
        return null;
    }
    getLastChildRoutes(path, routeList) {
        /*
          Recursive function that iterates throw a string composed of
          a set of action separated by '/' ex: 'action1/action2',
          and find if the action match with an entry. Then if it has childs,
          check if the childs can match with the next action.
        */
        if (!path)
            return null;
        var childRoutes = [];
        let [currentPath, ...childPath] = path.split('/');
        for (let r of routeList) { //iterate over all routeList
            if (r.action == currentPath)
                childRoutes = r.childRoutes;
            if (childRoutes && childRoutes.length && childPath.length > 0) {
                //evaluate childroute over next actions
                childRoutes = this.getLastChildRoutes(childPath.join('/'), childRoutes);
                if (childRoutes && childRoutes.length)
                    return childRoutes; //if we find return it
            }
            else if (childRoutes && childRoutes.length)
                return childRoutes; //last action and finded route
        }
        return null;
    }
    matchRoute(prop, matcher, input) {
        /*
          prop: ('text' | 'payload' | 'intent' | 'type' | ...)
          matcher: (string: exact match | regex: regular expression match | function: return true)
          input: user input object, ex: {type: 'text', data: 'Hi'}
        */
        let value = '';
        if (prop in input)
            value = input[prop];
        else if (prop == 'text')
            if (input.type == 'text')
                value = input.data;
        if (typeof matcher === 'string')
            return value == matcher;
        if (matcher instanceof RegExp)
            return matcher.exec(value);
        if (typeof matcher === 'function')
            return matcher(value);
        return false;
    }
    async processInput(input, routePath, context = {}) {
        if (input.type == 'text') {
            let intent = await this.getIntent(input);
            if (intent) {
                input.intent = intent.data.result.metadata.intentName;
                input.entities = intent.data.result.parameters;
            }
        }
        if (routePath)
            this.lastRoutePath = routePath;
        let { action, params } = this.getAction(input);
        let component = 'actions/' + action;
        const req = {
            headers: {},
            method: 'GET',
            url: component,
            input: input,
            context: context,
            params: params
        };
        const res = {};
        const pathname = component;
        const query = { routePath: this.lastRoutePath };
        return this.app.renderToHTML(req, res, pathname, query, {});
    }
    async getWebview(webview_name, context) {
        let component = path_1.join('/webviews', webview_name);
        const req = { headers: {}, method: 'GET', url: component, context: context };
        const res = {};
        const pathname = component;
        const query = {};
        return this.app.renderToHTML(req, res, pathname, query, {});
    }
    async getIntent(input) {
        if (this.conf.integrations && this.conf.integrations.dialogflow) {
            return axios_1.default({
                headers: {
                    Authorization: 'Bearer ' + this.conf.integrations.dialogflow.token
                },
                url: 'https://api.dialogflow.com/v1/query',
                params: {
                    query: input.data, lang: 'en', sessionId: this.df_session_id
                }
            });
        }
    }
}
exports.Botonic = Botonic;
class BotonicAPIService {
    constructor() {
        this.cliendId = BOTONIC_CLIENT_ID;
        this.clientSecret = BOTONIC_CLIENT_SECRET;
        this.baseUrl = BOTONIC_URL;
        this.baseApiUrl = this.baseUrl + '/v1/';
        this.loginUrl = this.baseUrl + '/o/token/';
        this.botPath = process.cwd();
        this.botCredentialsPath = path_1.join(this.botPath, '/.botonic.json');
        this.globalConfigPath = path_1.join(os_1.homedir(), '/.botonic');
        this.globalCredentialsPath = path_1.join(this.globalConfigPath, '/credentials.json');
        this.bot = null;
        this.headers = null;
        this.loadGlobalCredentials();
        this.loadBotCredentials();
    }
    beforeExit() {
        this.saveGlobalCredentials();
        this.saveBotCredentials();
    }
    loadGlobalCredentials() {
        try {
            var credentials = JSON.parse(fs.readFileSync(this.globalCredentialsPath, 'utf8'));
            var mixpanel = credentials.mixpanel ? credentials.mixpanel : credentials.me.campaign.mixpanel_id;
        }
        catch (e) { }
        if (credentials) {
            this.oauth = credentials.oauth;
            this.me = credentials.me;
            this.mixpanel = mixpanel;
            if (credentials.oauth)
                this.headers = {
                    Authorization: `Bearer ${this.oauth.access_token}`,
                    'content-type': 'application/json'
                };
        }
    }
    loadBotCredentials() {
        try {
            var credentials = JSON.parse(fs.readFileSync(this.botCredentialsPath, 'utf8'));
        }
        catch (e) { }
        if (credentials) {
            if (credentials.hasOwnProperty('bot')) {
                this.bot = credentials.bot;
                this.lastBuildHash = credentials.lastBuildHash;
            }
            else { // Allow users < v0.1.12 to upgrade smoothly
                this.bot = credentials;
                this.lastBuildHash = '';
            }
        }
    }
    async checkGlobalCredentialsPath() {
        if (!fs.existsSync(this.globalConfigPath))
            fs.mkdirSync(this.globalConfigPath);
    }
    async saveGlobalCredentials() {
        await this.checkGlobalCredentialsPath();
        fs.writeFileSync(this.globalCredentialsPath, JSON.stringify({
            oauth: this.oauth,
            me: this.me,
            mixpanel: this.mixpanel
        }));
    }
    saveBotCredentials() {
        let bc = { bot: this.bot, lastBuildHash: this.lastBuildHash };
        fs.writeFileSync(this.botCredentialsPath, JSON.stringify(bc));
    }
    async getCurrentBuildHash() {
        const options = {
            folders: { exclude: ['.*', 'node_modules'] },
            files: { include: ['*.js'] }
        };
        let hash = await hashElement('.', options);
        return hash.hash;
    }
    async build() {
        let spinner = new ora({
            text: 'Building...',
            spinner: 'bouncingBar'
        }).start();
        try {
            var build_out = await exec('npm run build');
        }
        catch (error) {
            spinner.fail();
            return false;
        }
        spinner.succeed();
        return true;
    }
    async buildIfChanged() {
        let hash = await this.getCurrentBuildHash();
        if (hash != this.lastBuildHash) {
            this.lastBuildHash = hash;
            return await this.build();
        }
        return true;
    }
    setCurrentBot(bot) {
        this.bot = bot;
    }
    setMixpanelInfo(mixpanel_id) {
        this.mixpanel = { 'mixpanel_id': mixpanel_id };
    }
    logout() {
        if (fs.existsSync(this.globalCredentialsPath))
            fs.unlinkSync(this.globalCredentialsPath);
    }
    async api(path, body = null, method = 'get', headers = null, params = null) {
        var b = 0;
        try {
            return await axios_1.default({
                method: method,
                url: this.baseApiUrl + path,
                headers: headers || this.headers,
                data: body,
                params: params
            });
        }
        catch (e) {
            b = 1;
        }
        if (b == 1) {
            await this.refreshToken();
        }
        return await axios_1.default({
            method: method,
            url: this.baseApiUrl + path,
            headers: headers || this.headers,
            data: body,
            params: params
        });
    }
    async refreshToken() {
        let resp = await axios_1.default({
            method: 'post',
            url: this.loginUrl,
            params: {
                'callback': 'none',
                'grant_type': 'refresh_token',
                'refresh_token': this.oauth.refresh_token,
                'client_id': this.cliendId,
                'client_secret': this.clientSecret
            }
        });
        if (!resp)
            return;
        this.oauth = resp.data;
        this.headers = { Authorization: `Bearer ${this.oauth.access_token}` };
        await this.saveGlobalCredentials();
        return resp;
    }
    async login(email, password) {
        let resp = await axios_1.default({
            method: 'post',
            url: this.loginUrl,
            params: {
                'username': email,
                'password': password,
                'client_id': this.cliendId,
                'client_secret': this.clientSecret,
                'grant_type': 'password'
            }
        });
        this.oauth = resp.data;
        this.headers = {
            Authorization: `Bearer ${this.oauth.access_token}`,
            'content-type': 'application/json'
        };
        resp = await this.api('users/me');
        if (resp)
            this.me = resp.data;
        return resp;
    }
    signup(email, password, org_name, campaign) {
        let url = `${this.baseApiUrl}users/`;
        if (campaign)
            campaign.mixpanel_id = this.mixpanel ? this.mixpanel.distinct_id : Math.round(Math.random() * 10000000000);
        let signup_data = { email, password, org_name, campaign };
        return axios_1.default({
            method: 'post',
            url: url,
            data: signup_data
        });
    }
    async saveBot(bot_name) {
        let resp = await this.api('bots/', { name: bot_name, framework: 'framework_botonic' }, 'post');
        if (resp.data)
            this.setCurrentBot(resp.data);
        return resp;
    }
    async getMe() {
        return this.api('users/me/');
    }
    async getBots() {
        return this.api('bots/', null, 'get', null, { organization_id: this.me.organization_id });
    }
    async getProviders() {
        return this.api('provider_accounts/', null, 'get', null, { bot_id: this.bot.id });
    }
    async deployBot(bundlePath, password) {
        try {
            let a = await this.getMe();
        }
        catch (e) {
            console.log(e);
        }
        const form = new FormData();
        let data = fs.createReadStream(bundlePath);
        form.append('bundle', data, 'botonic_bundle.zip');
        let headers = await this.getHeaders(form);
        return this.api(`bots/${this.bot.id}/deploy_botonic/`, form, 'post', Object.assign({}, this.headers, headers), { password: password });
    }
    async getHeaders(form) {
        //https://github.com/axios/axios/issues/1006#issuecomment-352071490
        return new Promise((resolve, reject) => {
            form.getLength((err, length) => {
                if (err) {
                    reject(err);
                }
                let headers = Object.assign({ 'Content-Length': length }, form.getHeaders());
                resolve(headers);
            });
        });
    }
}
exports.BotonicAPIService = BotonicAPIService;
