"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const command_1 = require("@oclif/command");
const botonic_1 = require("../botonic");
const utils_1 = require("../utils");
class Run extends command_1.Command {
    async run() {
        utils_1.track('botonic_input');
        const { args, flags } = this.parse(Run);
        const path = flags.path ? path_1.resolve(flags.path) : process.cwd();
        const route = flags.route ? flags.route : '';
        const context = flags.context ? flags.context : '';
        this.botonic = new botonic_1.Botonic(path);
        this.botonic.processInput(args.input, route, context).then((response) => {
            console.log(response);
        });
    }
}
Run.description = 'Get response from a single input';
Run.examples = [
    `$ botonic input "{\\"type\\": \\"text\\", \\"data\\": \\"hi\\"}"
Hello!
`,
];
Run.flags = {
    path: command_1.flags.string({ char: 'p', description: 'Path to botonic project. Defaults to current dir.' }),
    context: command_1.flags.string({ char: 'c', description: 'Context of current session', parse: JSON.parse }),
    route: command_1.flags.string({ char: 'r', description: 'Route of the current bot state.' })
};
Run.args = [{ name: 'input', parse: JSON.parse, required: true }];
exports.default = Run;
