"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const inquirer_1 = require("inquirer");
const command_1 = require("@oclif/command");
const botonic_1 = require("../botonic");
const utils_1 = require("../utils");
class Run extends command_1.Command {
    constructor() {
        super(...arguments);
        this.botonicApiService = new botonic_1.BotonicAPIService();
    }
    async run() {
        utils_1.track('botonic_log_in');
        const { args, flags } = this.parse(Run);
        const path = flags.path ? path_1.resolve(flags.path) : process.cwd();
        await this.logInUser();
    }
    askLoginInfo() {
        return inquirer_1.prompt([{
                type: 'input',
                name: 'email',
                message: 'email:'
            },
            {
                type: 'password',
                name: 'password',
                mask: '*',
                message: 'password:'
            }]);
    }
    async logInUser() {
        let user_data = await this.askLoginInfo();
        this.botonicApiService.login(user_data.email, user_data.password)
            .then((resp) => {
            if (resp.data.campaign && resp.data.campaign.mixpanel_id)
                this.botonicApiService.setMixpanelInfo(resp.data.campaign.mixpanel_id);
            this.botonicApiService.saveGlobalCredentials();
            console.log('Successful log in!'.green);
        }, (err) => {
            console.log('Error: '.red, err.response.data.error_description.red);
        });
    }
}
Run.description = 'Log in to Botonic';
Run.examples = [];
Run.flags = {
    path: command_1.flags.string({ char: 'p', description: 'Path to botonic project. Defaults to current dir.' })
};
Run.args = [];
exports.default = Run;
